define(["underscore"], function (_) {
    var Validation = function (model, data, $form) {
      this.model = model;
      this.data = data;
      this.$form = $form;
      this.success = false;
      this.error = false;
      this.config = {
        errorClass: "has-error"
      };
      var parent = this;

      this.successWrapper = function (model, res, xhr) {
        console.log("Validation.successWrapper", this, arguments);

        if (res === "ok" && parent.success) {
          parent.success(model, res, xhr);
        }
      };
      this.errorWrapper = function (model, xhr) {
        console.log("Validation.errorWrapper", this, arguments);

        // check for main error
        if (xhr.responseJSON && xhr.responseJSON.error) {
          var mainError = xhr.responseJSON.error;
          parent.$form.find("fieldset")
            .prepend('<p class="error help-block">'+mainError+'</p>');
        }

        // check for backend errors
        if (xhr.status === 404 && xhr.statusText) {
          var serverError = xhr.statusText;
          parent.$form.find("fieldset")
            .prepend('<p class="error help-block">Server Error: ' + serverError + '</p>');
        }

        // check for field errors
        if (xhr.responseJSON && xhr.responseJSON.errors) {
          var errors = xhr.responseJSON.errors;
          _.each(errors, function (field_errors, key) {

            // find input field by name
            var $input = parent.$form.find(":input[name="+key+"]");

            console.log("Validation.errorWrapper each->errors", key, field_errors, $input);

            // check if input field exists
            if ($input.length > 0) {
              // add parent form-group error class
              var $group = $input.parents(".form-group");
              $group.addClass(parent.config.errorClass);

              // display all error messages for current field
              _.each(field_errors, function (error) {
                $group.append('<p class="error help-block">'+error+'</p>');
              });
            }

          });
        }

        // focus first input field with error
        // parent.$form
        //   .find(parent.config.errorClass)
        //   .first()
        //   .find(":input")
        //   .focus();

        if (parent.error) {
          parent.error(model, xhr);
        }
      };
    };
    Validation.prototype.save = function (success, error) {

      if (this.$form.length === 0) {
        console.warn("Validation.$form not found", this.$form);
      }

      // remove previous errors
      this.$form.find(".error.help-block").remove();
      this.$form.find("."+this.config.errorClass)
        .removeClass(this.config.errorClass);

      this.success = success;
      this.error = error;
      var options = {error: this.errorWrapper, success: this.successWrapper};
      this.model.save(this.data, options);
      console.log("Validation.save", this, arguments);
    };
    return Validation;
});
