define([ 'jquery', 'underscore', 'backbone', 'handlebars', 'context', 'events', 'tpls'],
    function ($, _, Backbone, Handlebars, Context, Events, Templates) {
        return Backbone.View.extend({
            tagName: "li",
            // className is set dynamically at ConnectionsView
            // className: "connection-item",
            template: Templates["connection"],
            initialize: function () {
                _.bindAll(this, "removeConnection");
            },
            render: function () {
                this.$el.html(this.template({conn: this.model.toJSON(), url: this.model.abs_url()}));
                return this;
            },
            events: {
                "click": "activate",
                "click .remove": "removeConnection"
            },
            removeConnection: function (e) {

                // prevent parent event "activate"
                e.preventDefault();
                e.stopPropagation();

                if (window.confirm("Are you sure?")) {
                    Events.trigger("connection:remove", this.model);
                    this.model.destroy();
                    this.remove();
                }
            },
            activate: function (e) {
                e.preventDefault();

                // set context
                Context.connection = this.model.id;
                Context.driver = this.model.get("driver");
                Context.db = this.model.get("db");
                Context.table = null;
                Context.meta = {};

                // trigger event
                Events.trigger("connection:change", this.model);

                // check license when activating connection
                Events.trigger("license:check");
            }
        });
    });
