define(['jquery', 'underscore', 'backbone', 'tpls', 'collections/connections', 'views/connection',
        'events', 'context', 'router', 'models/license', 'views/update/menu', 'models/update',
        'models/feedback', 'tabs'],
    function ($, _, Backbone, Templates, Connections, ConnectionView, Events, Context,
              Router, License, UpdateMenuView, Update, Feedback, Tabs) {
        return Backbone.View.extend({
            template: Templates["connections"],
            tabConfig: {
              moreTabSelector: ".more-tab",
              // hidden tabs will be added to this drop down
              dropdownSelector: ".more-tab .dropdown-menu",
              // selector for tabs which should be readjusted
              tabSelector: "ul.nav-tabs>li:not(.action-tab)",
              // exclude elements from tabSelector
              excludeSelector: ".action-tab:visible"
            },
            initialize: function (opts) {

                _.bindAll(this, "add", "success", "navigateConnection");

                this.active = opts.active;
                this.connections = new Connections();

                this.listenTo(this.connections, 'add', this.add);
                this.listenTo(Events, 'connection:navigate', this.navigateConnection);
                this.listenTo(Events, 'db:dropped', this.reloadConnection);
                this.listenTo(Events, 'route:user:account', this.removeActiveClass);
                this.connections.fetch({success: this.success});

            },
            events: {
                "click li.connection-item": function (e) {
                    // $(e.currentTarget).siblings().removeClass("active");
                    this.$("li.active").removeClass("active");
                    $(e.currentTarget).addClass("active");
                },
                "click .create-new-connection": function (e) {
                    $(e.currentTarget).closest('li').siblings().removeClass("active");
                    // don't prevent default, because we need to update the url
                    //e.preventDefault();
                    Events.trigger("connection:new");
                },
                "click .feedback-menu>a": "feedbackModal",
                "submit .feedback-modal form": "sendFeedback",
                "show.bs.dropdown .dropdown": function (e) {
                  // by default tab-bar must be overflow:hidden, because
                  // toomanytabs has resize timeout and the adjustment doesnt
                  // happen instantly. it means that for a small moment
                  // tabs get overflow over and it breaks ui
                  // so we are setting the overflow to visible only when user
                  // actually needs to see the dropdown

                  $(e.target).parents(".tab-bar").css("overflow", "visible");
                },
                "hide.bs.dropdown .dropdown": function (e) {
                  // return tab-bar back to normal state
                  $(e.target).parents(".tab-bar").css("overflow", "hidden");
                }
            },
            removeActiveClass: function () {
                this.$("li.active").removeClass("active");
            },
            reloadConnection: function () {
                this.navigateConnection(Context.connection);
            },
            navigateConnection: function (id) {
                var $conn = this.$("span.connection").filter(function () {
                    return parseInt($(this).data("id")) === parseInt(id);
                });
                $conn.click();
            },
            add: function (connection) {
                var view = new ConnectionView({model: connection, className: this.classNameCbk(connection)});
                view.render().$el.insertAfter(this.$(".logo-wrapper"));
            },
            classNameCbk: function (conn) {
                if (conn.get('id') === Context.connection) {
                    return "connection-item active";
                }
                return "connection-item";
            },
            tooManyTabs: function () {
                _.defer(function () {
                    this.tooManyTabs = this.$(".nav-tabs").tooManyTabs(this.tabConfig);
                });
            },
            checkLicense: function () {
                var $license_item = $("<li>").addClass("license-item")
                    .append($("<a>").attr("href", "#/license").text("License"));

                $license_item.insertAfter(this.$(".user-menu .dropdown-menu>li:first"));

                // request license check
                Events.trigger("license:check");
            },
            success: function (model, data) {
                this.tooManyTabs();

                Context.connections = data;
                Events.trigger("connections:fetched");

                if (Context.isAdmin()) {
                  // trigger license and update check
                  this.checkLicense();
                  this.checkUpdates();
                  this.$(".admin-item").show();
                }
            },
            checkUpdates: function () {
                var update = new Update();
                update.fetch({success: this.addUpdate});
            },
            addUpdate: function (model) {
                if (model.get("new_update")) {
                    var view = new UpdateMenuView({model: model});
                    view.render().$el.insertAfter(this.$(".user-menu"));
                }
            },
            feedbackModal: function (e) {
                if (e) {
                    e.preventDefault();
                }

                this.$(".feedback-modal").modal();
                this.$(".feedback-modal input:first").focus();
                this.$(".feedback-modal .success-msg").hide();
            },
            sendFeedback: function (e) {
                if (e) {
                    e.preventDefault();
                }
                var $body = this.$(".feedback-modal form textarea[name=body]");

                var $msg = this.$(".feedback-modal .success-msg");
                $msg.hide();

                var $btn = this.$(".feedback-modal .btn-send");
                $btn.text("Sending...").prop("disabled", true);

                var data = {};
                data.body = $body.val();

                if (data.body.length === 0) {
                    return;
                }

                var feedback = new Feedback();
                feedback.save(data, {
                    success: function () {
                        $msg.show();
                        $body.val("");
                        $btn.text("Send").prop("disabled", false);
                    },
                    error: function (model, xhr) {
                        var error = "Oops, failed to submit feedback.";
                        if (xhr.responseJSON && xhr.responseJSON.error) {
                            error = xhr.responseJSON.error;
                        }
                        alert(error);
                        $btn.text("Send").prop("disabled", false);
                    }
                });


            },
            render: function () {
                this.$el.html(this.template({active: this.active}));
                this.$(".admin-item").hide();
                return this;
            }
        });
    });
