define(["jquery", "underscore", "backbone", "tpls", "context", "events", "collections/backups", "views/content/backup", "baseview", "models/backup"],
	function($, _, Backbone, Templates, Context, Events, Backups, BackupView, BaseView, Backup) {
		var BackupsView = BaseView.extend({
			template: Templates["backups/index"],
			emptyRow: "<tr><td colspan=99>No backups found.</td></tr>",
			initialize: function() {

				_.bindAll(this, "fetchSuccess", "fetchError", "addBackup", "reloadBackups");

				this.backups = new Backups();

				// listen to global events
				this.listenTo(Events, "db:change", this.render, this);
				this.listenTo(Events, "backup:delete", this.reloadBackups, this);
				this.listenTo(Events, "backup:finished", this.reloadBackups, this);

				// listen to websockets for backup state changes
				var wsurl = "ws://" + location.hostname + (location.port ? ":" + location.port : "") + "/ws";
				var ws = new WebSocket(wsurl);

				console.log("connecting to websocket service for backup state changes at", wsurl);

        // when receiving websocket message, parse it and trigger backup:finished event
				ws.onmessage = function(res) {
					try {

            // try to parse json data from websocket message
						var data = JSON.parse(res.data);
						console.log("response from backup websocket service", data);

            // if data matches needed format, trigger event to reload backup view
            // @todo check if backup.id is in the current list of backups
						if (data.type === "backup" && data.status === "finished" && data.backup) {
							Events.trigger("backup:finished", data.backup);
						}

					} catch (e) {
            // log parse errors
						console.warn("warning: couldn't parse websocket json response");
					}
				};

        // log websocket errors
				ws.onerror = function(message) {
					console.warn("websocket error:", message);
				};
			},
			events: {
				"submit .create-backup-form": "createBackup"
			},
      reloadBackups: function () {
        console.log("reload backups");
        this.backups.fetch({
          reset: true,
          success: this.fetchSuccess,
          error: this.fetchError
        });
      },
			createBackup: function(e) {
				e.preventDefault();

				// this.backups.create(data);
				// this.backups.create(data, {success: function (model, res, xhr) {
				//     // that.render();
				// }});

				// serialize form data for backup
				var data = Context.serializeObj($(e.target));

				// new instance of backup model
				var backup = new Backup();

				// save backup
				var that = this;
				backup.save(data, {
					success: function(model, res, xhr) {
						console.log("BackupsView.createBackup", arguments);

						// reload list of backups completely
						// that.backups = new Backups();
						that.reloadBackups();
					}
				});
			},
			render: function() {
				this.$el.html(this.template({
					Context: Context
				}));

        // disable backup creation for non-mysql dbs
        if (!Context.isMySQL()) {
          this.$(".btn-primary").prop("disabled", true);
        } else {
          this.$(".btn-primary").prop("disabled", false);
        }

				this.backups.fetch({
					reset: true,
					success: this.fetchSuccess,
					error: this.fetchError
				});
				return this;
			},
			addBackup: function(backup) {
				this.subviews[backup.cid] = new BackupView({
					model: backup
				});
				this.$("tbody").append(this.subviews[backup.cid].render().el);
			},
			fetchSuccess: function(models, res, xhr) {
				console.log("BackupsView.fetchSuccess", arguments, this.backups);

				this.$("tbody").empty();
				models.each(this.addBackup);
				if (models.length === 0) {
					this.$("tbody").html(this.emptyRow);
				}

				// if (this.backups.length > 0) {
				//     this.$(".empty-row").remove();
				// }
				// this.backups.sort();
			},
			/**
			 * Handle backup fetch errors
			 */
			fetchError: function() {
				alert("Failed to fetch backups!");
			}
		});
		return BackupsView;
	});
