define(['jquery', 'underscore', 'backbone', 'tpls', 'context'],
    function ($, _, Backbone, Templates, Context) {
      var ConditionView = Backbone.View.extend({
        className: "row filter-condition-row",
        template: Templates["content/query/filter/condition"],
        events: {
          'click .remove': 'removeView',
          'change :input': 'change'
        },
        initialize: function () {
          _.bindAll(this, "focus");
        },
        removeView: function () {
          this.model.destroy();
          this.remove();
        },
        change: function (e) {
          var $input = $(e.currentTarget);
          this.model.set($input.attr('name'), $input.val());

          // disable/enable value field if operator is not IS NULL/IS NOT NULL
          this.$(":input[name=value]").prop("disabled", this.model.isOpNully());
        },
        focus: function () {
          this.$("input[type=text],textarea,select").first().focus();
        },
        /**
         * Set input values back to model
         */
        sync: function (model) {
          this.$(":input").each(function () {
            var $input = $(this);
            if ($input.attr("name")) {
              model.set($input.attr("name"), $input.val())
            }
          });
        },
        render: function () {
          var data = {
            model: this.model.toJSON(),
            columns: Context.meta.columns
          };
          var tpl = this.template(data);
          this.$el.html(tpl);

          this.sync(this.model);

          return this;
        }
      });
      return ConditionView;
    });