define(["jquery", "underscore", "backbone", "tpls", "context", "grid", "models/table/schema", "events", "models/table/column", "models/table/index", "helpers/validation", "models/table/foreignkey", "rowactions", "models/types"],
	function($, _, Backbone, Templates, Context, Grid, Schema, Events, Column,
    Index, Validation, ForeignKey, RowActions, Types) {
		return Backbone.View.extend({
			template: Templates["schema/main"],
			gridSettings: {
				tableClass: "table table-bordered table-condensed",
				sortable: false,
				emptyRow: true
			},
			disabledGridSettings: {
				tableClass: "table table-bordered table-condensed",
				sortable: false,
				emptyRow: false,
				readonly: true,
				emptyGridMessage: "No data is available."
			},
			table: null,
      indexTypes: {
        mysql: ["index", "primary", "unique", "fulltext"],
        postgresql: ["index", "unique"],
        redshift: ["index", "unique"]
      },
      fkActions: {
        mysql: ["restrict", "cascade", "set null", "no action"],
        postgresql: [],
        redshift: []
      },
			columns: ["Field", "Type", "CharacterSet", "Null", "Default", "Extra", "Comment"],
			gridColumns: function() {

				this.charsets.unshift("");

				var charsets = {
					values: this.charsets
				};

				var nulls = {
					values: ["YES", "NO"]
				};

        var typesModel = new Types();
        var types = {
          values: typesModel.getCurrentTypeValues()
        };

				return _.map(this.columns, function(col) {
					if (col === "CharacterSet") {
						return {
							name: col,
							type: "string",
							editor: "SelectEditor",
              display: "Encoding",
							editorProps: charsets
						};
          } else if (col === "Null") {
            return {
              name: col,
              type: "string",
              editor: "BooleanEditor"
              //editorProps: nulls
            };
          } else if (col === "Unsigned" || col === "Zerofill") {
            return {
              name: col,
              type: "string",
              style: function (value, rowData, $cell, grid) {

                // if (_.isEmpty(rowData)) {
                //   return null;
                // }

                // if (_.isEmpty(value)) {
                //   return {"color":"#ccc"};
                // }

                return null;
              },
              editor: function (grid) {
                // var $td = grid.getActiveCell();
                // var data = grid.getCellData($td);

                // if (_.isEmpty(data)) {
                //   return "DisabledEditor";
                // }

                return "BooleanEditor";
              }
            };
          } else if (col === "Length") {
            return {
              name: col,
              type: "string",
              editor: "BasicEditor",
              display: "Length/Param"
            };
          } else if (col === "Type") {
            return {
              name: col,
              type: "string",
              editor: "SelectEditor",
              editorProps: types
            };
					} else {
						return {
							name: col,
							type: "string",
							editor: "BasicEditor"
						};
					}
				});
			},
			indexColumns: {
        redshift: [{
          name: "Name",
          type: "string"
        }, {
          //   name: "Type",
          //   type: "string"
          // }, {
          name: "Columns",
          type: "string"
        }, {
        //  name: "Size",
        //  type: "string"
        //}, {
        //  name: "Owner",
        //  type: "string"
        //}, {
        //  name: "Unique",
        //  type: "boolean"
        //}, {
        //  name: "Description",
        //  type: "string"
        //}, {
          name: "Definition",
          type: "string",
          wrap: true
        }],
				postgresql: [{
					name: "Name",
					type: "string"
				}, {
					//   name: "Type",
					//   type: "string"
					// }, {
					name: "Columns",
					type: "string"
				}, {
					name: "Size",
					type: "string"
				}, {
					name: "Owner",
					type: "string"
				}, {
					name: "Unique",
					type: "boolean"
				}, {
					name: "Description",
					type: "string"
				}, {
					name: "Definition",
					type: "string",
					wrap: true
				}],
				mysql: [{
					name: "Name",
					type: "string"
				}, {
					name: "Type",
					type: "string"
				}, {
					name: "Columns",
					type: "string"
				}, {
					name: "Sub_Part",
					type: "string",
					display: "Sub Part"
				}, {
					name: "Unique",
					type: "boolean"
				}, {
					name: "Comment",
					type: "string"
				}]
			},
			events: {
				"click .inner-menu .btn": "activateView",
				"click .create-index": "createIndexPopup",
				"submit .create-index-modal form": "createIndex",
				"click .create-fk": "createFKPopup",
				"submit .create-fk-modal form": "createFK",
        "change .create-fk-modal select[name=referenced_table]":
          "updateFKRefColumns"
			},
			initialize: function() {
				_.bindAll(this, "updateColumn", "createColumn", "renderGrid", "error",
          "changeError", "changeSuccess", "setTable", "activateView",
          "resetView", "fetchSchema", "updateFKRefColumns");

				// listen to global events
				this.listenTo(Events, "table:change", this.setTable);
				this.listenTo(Events, "db:change", this.resetView);
			},
			setTable: function(table) {
				this.fetchSchema();
			},
			resetView: function() {
        var types = [];
        if (this.indexTypes[Context.driver]) {
          types = this.indexTypes[Context.driver];
        }

        var fkActions = [];
        if (this.fkActions[Context.driver]) {
          fkActions = this.fkActions[Context.driver];
        }

				this.$el.html(this.template({types: types, fkActions: fkActions}));
			},
			hideViews: function() {
				this.$(".index-wrapper, .foreignkey-wrapper, .structure-wrapper, .trigger-wrapper").hide();
			},
			fetchSchema: function() {
				// if (Context.table !== this.table) {
					this.$(".alert").remove();
					var schema = new Schema();
					schema.fetch({
						success: this.renderGrid,
						error: this.error
					});
					this.table = Context.table;
				// }
			},
			createIndexPopup: function(e) {
				this.$(".create-index-modal").modal();
				this.$(".create-index-modal :input:first").focus();
			},
			createIndex: function(e) {
				e.preventDefault();

        // serialize form data
        var data = Context.serializeObj($(e.currentTarget));

        // reset columns because they need manual processing
        data.columns = null;

        // find columns input
        var $select = $(e.currentTarget).find("select[name=columns]");

        // check if anything is selected in columns input
        if ($select.val() && $select.val().length > 0) {
          // join selected columns together in comma separated string
          data.columns = $select.val().join(",");
        }

        // create new instance of index model
        var index = new Index();
        var parent = this;

        var v = new Validation(index, data, this.$(".create-index-modal form"));
        v.save(function () {
          parent.$(".create-index-modal").modal("hide");
          parent.fetchSchema();
        }, function () {console.log("custom error");});
			},
      updateFKRefColumns: function (e) {
        // referenced_table is changed in fk popup, need to reload columns
        var $cols = this.$(".create-fk-modal select[name=referenced_columns]");
        var $table = this.$(".create-fk-modal select[name=referenced_table]");
        $cols.html("<option disabled=disabled>Loading...</option>");

        var table = $table.val();

        if (_.isEmpty(table)) {
          console.log("StructureView.updateFKRefColumns",
            "referenced_table not selected")
          return false;
        }

        // new schema instance for referenced_table
        var refSchema = new Schema();

        // create copy of context and update table to referenced_table
        var localContext = _.clone(Context);
        localContext.table = table;
        refSchema.setContext(localContext);

        // fetch schema of referenced_table
        refSchema.fetch({
          success: function (model, data, xhr) {
            console.log("StructureView.updateFKRefColumns",
              "refSchema fetched", data);

            // extract columns names
            var columns = _.pluck(data.meta, "Field");

            // update referenced_columns according to newnly fetched schema
            $cols.html(null);
            _.each(columns, function (column) {
              $cols.append('<option value="%1$s">%1$s</option>'.format(column));
            });
          },
          error: function () {
            // schema fetch failed, show error
            var err = "Error: failed to fetch referenced columns for referenced table: " + table;
            console.warn(err);
            alert(err);
          }
        });
      },
			createFKPopup: function(e) {
				this.$(".create-fk-modal").modal();
				this.$(".create-fk-modal :input:first").focus();
			},
			createFK: function(e) {
				e.preventDefault();

        // serialize form data
        var data = Context.serializeObj($(e.currentTarget));

        // reset columns because they need manual processing
        data.columns = null;

        // find columns input
        var $select = $(e.currentTarget).find("select[name=columns]");

        // check if anything is selected in columns input
        if ($select.val() && $select.val().length > 0) {
          // join selected columns together in comma separated string
          data.columns = $select.val().join(",");
        }

        // create new instance of index model
        var fk = new ForeignKey();
        var parent = this;

        console.log("ForeignKey.save", Context, fk);

        var v = new Validation(fk, data, this.$(".create-fk-modal form"));
        v.save(function () {
          parent.$(".create-fk-modal").modal("hide");
          parent.fetchSchema();
        }, function () {console.log("custom error");});
			},
			activateView: function(e) {

				// set active view name
				var view = Context.schemaView || "structure";
				if (e && e.target) {
					view = $(e.target).data("view");
				}

				// update context, schemaView is remembered between table changes
				Context.schemaView = view;

				// activate correct menu item, needed because of programmatic access
				// to the activateView method, no real click is triggered on startup
				this.$(".inner-menu .btn").removeClass("active");
				this.$(".inner-menu .btn[data-view=" + view + "]").addClass("active");

				// show correct view and hide others
				this.hideViews();
				this.$(".%s-wrapper".format(view)).show();

				// toggle actions for current view
				this.$(".schema-actions .action-group").hide();
        var sel = ".schema-actions .action-group[data-view=%s]".format(view);
				this.$(sel).show();
			},
			render: function() {

				Events.trigger("builder:render");

				this.resetView();

				this.$(".choose-table-placeholder").show();
				this.$(".schema-content").hide();

        // no need to call fetchSchema here, already called by setTable
        // otherwise results in double grid render
				// this.fetchSchema();

				return this;
			},
			/**
			 * Handle schema fetch errors
			 * @param model
			 * @param error
			 */
			error: function(model, error) {
				alert("Failed to fetch table schema.");
			},
      updateFormModals: function (data) {

        // if currently using postgres, hide action buttons
        var $buttons = this.$(".create-index, .create-fk");
        if (!Context.isMySQL()) {
          $buttons.hide();
        } else {
          $buttons.show();
        }

        // reset modals name inputs
        this.$(".create-index-modal, .create-fk-modal")
          .find("input[name=name]").val(null);

        // reset column selects
        var $select = this.$(".create-index-modal select[name=columns], .create-fk-modal select[name=columns]");
        $select.html(null);

        // reset referenced table select
        var $refs = this.$(".create-fk-modal select[name=referenced_table]");
        $refs.html(null);

        // need to take columns from Context.meta.columns because data.columns
        // are used for structure grid itself
        if (!_.isEmpty(data.meta)) {

          // load columns
          var columns = _.pluck(data.meta, "Field");
          _.each(columns, function (column) {
            $select.append('<option value="%1$s">%1$s</option>'.format(column));
          });

          // load tables into fk modal
          _.each(Context.tables, function (table) {
            // only show tables other than the active one
            if (table !== Context.table) {
              $refs.append('<option value="%1$s">%1$s</option>'.format(table));
            }
          });

          // trigger fk ref column fetch
          this.updateFKRefColumns();
        }

      },
			/**
			 * Render sensei grid
			 * @param model
			 * @param data
			 */
			renderGrid: function(model, data) {

        // schema is finally loaded, update modals
        this.updateFormModals(data);

				// this.$(".index-wrapper, .foreignkey-wrapper").hide();
				this.$(".choose-table-placeholder").hide();
				this.$(".schema-content").show();

				// store initial data for later use
				this.initialData = data.meta;
				this.charsets = data.charsets;
				this.columns = data.columns;
				this.indexes = data.indexes;
				this.foreignKeys = data.foreignKeys;
				this.triggers = data.triggers;

				// get sensei grid container element
				var $el = this.$(".sensei-grid-structure");

				// grid initialization
				this.grid = $el.grid(
					this.initialData,
					this.gridColumns(),
					this.gridSettings
				);

				// register basic editor
				this.grid.registerEditor(BasicEditor);
				this.grid.registerEditor(SelectEditor);
        this.grid.registerEditor(BooleanEditor);
				this.grid.registerEditor(DisabledEditor);

				// listen to events
				this.grid.events.on("editor:save", this.updateColumn);
				this.grid.events.on("row:save", this.createColumn);
				this.grid.events.on("row:remove", this.deleteColumn);

				// render grid
				this.grid.render();

				// render index grid
				this.renderIndexGrid();

				// render fk grid
				this.renderForeignKeyGrid();

				// trigger view is currently supported for mysql only
				if (Context.isMySQL()) {
					// render trigger grid
					this.renderTriggerGrid();
				}

				// activate default view
				this.activateView();
			},
			renderIndexGrid: function(model, data) {

				// get sensei grid container element
				var $el = this.$(".sensei-grid-index");

				// set correct columns for db driver
				var columns = [];
				if (this.indexColumns[Context.driver]) {
					columns = this.indexColumns[Context.driver];
				}

				// grid initialization
				this.indexgrid = $el.grid(this.indexes, columns, this.disabledGridSettings);

        var IndexDeleteRowAction = RowAction.extend({
      		name: "IndexDeleteRowAction",
          rowElement: function () {
            return "<button class='btn btn-danger btn-xs'>Delete</button>";
          },
          triggerEvent: {event: "click", selector: ".btn"},
          trigger: function (e) {
            var $activeCell = e.data.$activeCell;
            console.log("IndexDeleteRowAction.trigger", $activeCell);

            // get button from active cell
            var $btn = $activeCell.find(".btn");

            // simulate some loading action on demo button
            if (!$btn.prop("disabled")) {
              $btn.prop("disabled", true)
                .text("Deleting...");

              var grid = this.grid;
              var $row = this.grid.getCellRow($activeCell);
              var rowData = this.grid.getRowData($row);

              console.log("pre drop index", rowData);

              var index = new Index({name: rowData.Name});
              index.idAttribute = "name";
              index.destroy({
                success: function () {
                    grid.removeRow($row);
                },
                error: function (model, xhr) {
                  // reset button
                  $btn.prop("disabled", false).text("Delete");

                  // default error message
                  var error = "Failed to remove index";

                  // try to get error from json
                  if (xhr.responseJSON && xhr.responseJSON.error) {
                    error = xhr.responseJSON.error;
                  }

                  // log and display error
                  console.warn(error, arguments);
                  alert(error);
                }
              });
            }
          }
      	});

        if (Context.isMySQL()) {
          this.indexgrid.registerRowAction(IndexDeleteRowAction);
        }

				// register basic editor
				// this.indexgrid.registerEditor(BasicEditor);
				// this.indexgrid.registerEditor(SelectEditor);
				// this.indexgrid.registerEditor(BooleanEditor);

				// this.indexgrid.events.on("editor:save", function() {
				// 	console.log("indexgrid editor:save")
				// });

				// render grid
				this.indexgrid.render();
			},
			renderForeignKeyGrid: function(model, data) {

				// get sensei grid container element
				var $el = this.$(".sensei-grid-fk");

				/**
        **Example data**
        Column: "author_id"
        Constraint: "author_id_refs_id_8a197f2f"
        Referenced_Column: "id"
        Referenced_Table: "auth_user"
        Table: "vellum_post"
        */

				// grid initialization
				this.fkgrid = $el.grid(
					this.foreignKeys, [{
						name: "Constraint",
						type: "string",
						display: "Name"
					}, {
						name: "Table",
						type: "string"
					}, {
						name: "Column",
						type: "string"
					}, {
						name: "Referenced_Column",
						type: "string"
					}, {
						name: "Referenced_Table",
						type: "string"
					}],
					this.disabledGridSettings);

        var FKDeleteRowAction = RowAction.extend({
      		name: "FKDeleteRowAction",
          rowElement: function () {
            return "<button class='btn btn-danger btn-xs'>Delete</button>";
          },
          triggerEvent: {event: "click", selector: ".btn"},
          trigger: function (e) {
            var $activeCell = e.data.$activeCell;
            console.log("FKDeleteRowAction.trigger", $activeCell);

            // get button from active cell
            var $btn = $activeCell.find(".btn");

            // simulate some loading action on demo button
            if (!$btn.prop("disabled")) {
              $btn.prop("disabled", true)
                .text("Deleting...");

              var grid = this.grid;
              var $row = this.grid.getCellRow($activeCell);
              var rowData = this.grid.getRowData($row);

              var fk = new ForeignKey({name: rowData.Constraint});
              fk.idAttribute = "name";
              fk.destroy({
                success: function () {
                  console.log("fk.destroy", arguments);
                  grid.removeRow($row);
                },
                error: function (model, xhr) {
                  // reset button
                  $btn.prop("disabled", false).text("Delete");

                  // default error message
                  var error = "Failed to remove foreign key";

                  // try to get error from json
                  if (xhr.responseJSON && xhr.responseJSON.error) {
                    error = xhr.responseJSON.error;
                  }

                  // log and display error
                  console.warn(error, arguments);
                  alert(error);
                }
              });


              // setTimeout(function () {
              //   $btn.addClass("btn-danger")
              //     .text("Deleted");
              //   grid.removeRow($row);
              // }, 500);
            }
          }
      	});

        if (Context.isMySQL()) {
          this.fkgrid.registerRowAction(FKDeleteRowAction);
        }

				// render grid
				this.fkgrid.render();
			},
			renderTriggerGrid: function(model, data) {

				// get sensei grid container element
				var $el = this.$(".sensei-grid-trigger");

				/**
        **Example data**
        Created: null
        Database Collation: "utf8_general_ci"
        Definer: "root@localhost"
        Event: "INSERT"
        Statement: "set names utf8;"
        Table: "test_table"
        Timing: "AFTER"
        Trigger: "test_trigger"
        character_set_client: "utf8"
        collation_connection: "utf8_general_ci"
        sql_mode: "STRICT_TRANS_TABLES,NO_ENGINE_SUBSTITUTION"
        */

				// grid initialization
				this.triggergrid = $el.grid(
					this.triggers, [{
						name: "Event",
						type: "string"
					}, {
						name: "Statement",
						type: "string"
					}, {
						name: "Timing",
						type: "string"
					}, {
						name: "Trigger",
						type: "string"
					}, {
						name: "sql_mode",
						type: "string"
					}],
					this.disabledGridSettings);

				// this.triggergrid.events.on("editor:save", function() {
				// 	console.log("fk editor:save")
				// });
				//
				// // register basic editor
				// this.triggergrid.registerEditor(BasicEditor);
				// this.triggergrid.registerEditor(SelectEditor);
				// this.triggergrid.registerEditor(BooleanEditor);

				// render grid
				this.triggergrid.render();
			},
			/**
			 * Update an existing column
			 * @param data
			 * @param $cell
			 */
			updateColumn: function(data, $cell) {

				if (!this.grid.getCellStatus($cell)) {
					console.log("nope updateColumn");
					return;
				}

				console.log("updateColumn", data, $cell);
				_.forIn(data, function(value, param) {
					var $row = this.grid.getCellRow($cell);
					var index = $row.index();

					if (this.initialData[index] && this.initialData[index]["Field"]) {

						var column = this.initialData[index]["Field"];
						var data = {
							param: param,
							value: value,
							column: column
						};
						var schema = new Schema();

						schema.save(data, {
							success: this.changeSuccess,
							error: this.changeError
						});
					} else {
						this.changeError();
					}
				}, this);
			},
			/**
			 * Create a new column
			 * @param data
			 * @param $row
			 */
			createColumn: function(data, $row) {
				console.log("create column", data, $row);
				// a little workaround for backbone
				var schema = new Schema();
				schema.setCreateURL();
				var grid = this.grid;
				schema.save(data, {
					success: function(model, response, options) {
						grid.setRowSaved($row);
						Events.trigger("schema:changed");
					},
					error: this.changeError
				});
			},
			/**
			 * Delete a column
			 * @param data
			 * @param row
			 * @param $row
			 */
			deleteColumn: function(data, row, $row) {
				console.log("StructureView.deleteColumn", data, $row);

				var col = new Column({
					title: data["Field"]
				});
				col.destroy({
					success: function() {
						console.log("column dropped", arguments, col);
						Events.trigger("column:dropped", col);
						Events.trigger("schema:changed");
					},
					error: function() {
						alert("Unexpected error: failed to drop column.");
						console.log("StructureView.deleteColumn Error", arguments);
					}
				});
			},
			changeError: function(model, response, options) {
				if (response["responseJSON"] && response["responseJSON"]["error"]) {
					console.log(response);
					alert("Schema update failed. " + response["responseJSON"]["error"]);
				} else {
					alert("Schema update failed: unexpected error.");
				}
			},
			changeSuccess: function(model, response, options) {
				// do nothing for now
				Events.trigger("schema:changed");
			}
		});
	});
